/***************************************************************************
 *            gnc-object.c
 *
 *  mié ene 10 12:23:52 2007
 *  Copyright  2007  Daniel Espinosa Ortiz
 *  esodan@gmail.com
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gnc-object.h"
#include "gnc-engine.h"
 
static void gnc_object_class_init(GncObjectClass *klass);
static void gnc_object_init(GncObject *sp);
static void gnc_object_finalize(GObject *object);

struct _GncObjectPrivate {
	/* Private Members */
};

typedef struct _GncObjectSignal GncObjectSignal;
typedef enum _GncObjectSignalType GncObjectSignalType;

enum _GncObjectSignalType {
	/* Signals */
	LAST_SIGNAL
};

/* properties */
enum
{
        PROP_0,
	PROP_QOF_BOOK
};

struct _GncObjectSignal {
	GncObject *object;
};

static guint gnc_object_signals[LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

GType
gnc_object_get_type()
{
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo our_info = {
			sizeof (GncObjectClass),
			NULL,
			NULL,
			(GClassInitFunc)gnc_object_class_init,
			NULL,
			NULL,
			sizeof (GncObject),
			0,
			(GInstanceInitFunc)gnc_object_init,
		};

		type = g_type_register_static(G_TYPE_OBJECT, 
			"GncObject", &our_info, 0);
	}

	return type;
}

static void
gnc_object_class_init(GncObjectClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS(klass);

	parent_class = g_type_class_peek_parent(klass);
	object_class->finalize = gnc_object_finalize;
	object_class->set_property = gnc_object_set_property;
//    object_class->get_property = gnc_object_get_property;

	/* Install properties */
	g_object_class_install_property (object_class, PROP_QOF_BOOK,
					 g_param_spec_pointer ("qof_book", 
							       _("Pointer to a QofBook"), NULL,
							       G_PARAM_WRITABLE));
	/* Create signals here:
	   gnc_object_signals[SIGNAL_TYPE_EXAMPLE] = g_signal_new(...)
 	*/
 	
}

static void
gnc_object_init(GncObject *obj)
{
	obj->priv = g_new0(GncObjectPrivate, 1);
	/* Initialize private members, etc. */
}

static void
gnc_object_finalize(GObject *object)
{
	GncObject *cobj;
	cobj = GNC_OBJECT(object);
	
	/* Free private members, etc. */
		
	g_free(cobj->priv);
	G_OBJECT_CLASS(parent_class)->finalize(object);
}

static void
gnc_object_set_property (GObject *object,
				  guint param_id,
				  const GValue *value,
				  GParamSpec *pspec)
{
	GncObject *obj;
	QofBook *book;
	
	obj = GNC_OBJECT (object);
	switch (param_id) {
		case PROP_QOF_BOOK:
			book = (QofBook*) g_value_get_pointer (value);
			gnc_object_set_qof_book (obj, book);
		break;
		
		default:
   			/* We don't have any other property... */
    		G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    	break;
	}
}

static void
gnc_object_get_property (GObject      *object,
                        guint         property_id,
                        GValue       *value,
                        GParamSpec   *pspec)
{
  GncObject *obj = GNC_OBJECT(object);

  switch (property_id) {
  	
  	case PROP_QOF_BOOK: {
    	g_value_set_pointer (value, gnc_object_get_qof_book (object) );
  	}
    break;
    
  default:
    /* We don't have any other property... */
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    break;
  }
}


/**
 * gnc_object_new
 *
 * Creates a new #GncObject.

 * Returns: a newly created #GncObject.
 */
GncObject *
gnc_object_new()
{
	GncObject *obj;
	
	obj = GNC_OBJECT (g_object_new(GNC_TYPE_OBJECT, NULL));
	
	return obj;
}

/**
 * gnc_object_set_qof_book
 *
 * @book: A QofBook
 *
 * Sets the book property using a @QofBook.
 */
 void
 gnc_object_set_qof_book (GncObject *object, QofBook *book)
 {
 	g_return_if_fail (GNC_IS_OBJECT (object));
 	g_return_if_fail (book == NULL);
	
	if (GNC_OBJECT_GET_CLASS (model)->set_qof_book)
		(GNC_OBJECT_GET_CLASS (model)->set_qof_book) (object, book);
	else
		g_warning ("%s() method not supported\n", __FUNCTION__);
 }

/**
 * gnc_object_get_qof_book
 *
 * @object: A GncObject
 *
 * Gets the book property in the @GncObject.
 *
 * Returns: a @QofBook.
 *
 */
 void
 gnc_object_get_qof_book (GncObject *object)
 {
 	g_return_if_fail (GNC_IS_OBJECT (object));
	
	if (GNC_OBJECT_GET_CLASS (model)->get_qof_book)
		(GNC_OBJECT_GET_CLASS (model)->get_qof_book) (object);
	else
		g_warning ("%s() method not supported\n", __FUNCTION__);
 }

/* GType Definitions */

/**
 * gnc_numeric_copy
 * @src: source to get a copy from.
 *
 * Creates a new #GncNumeric structure from an existing one.

 * Returns: a newly allocated #GncNumeric which contains a copy of
 * information in @src.
 */
gpointer
gnc_numeric_copy (gpointer boxed)
{
	GncNumeric *src = (GncNumeric*) boxed;
	GncNumeric *copy = NULL;

	g_return_val_if_fail (src, NULL);

	copy = g_new0 (GncNumeric, 1);
	copy->num = src->num;
	copy->denom = src->denom;

	return copy;
}

/**
 * gda_binary_free
 * @boxed: #GdaBinary to free.
 *
 * Deallocates all memory associated to the given #GdaBinary.
 */
void
gda_binary_free (gpointer boxed)
{
	GdaBinary *binary = (GdaBinary*) boxed;
	
	g_return_if_fail (binary);
		
	g_free (binary->data);
}

static void 
string_to_gnc_numeric (const GValue *src, GValue *dest) 
{
	/* FIXME: add more checks*/
	GncNumeric *numeric;
	const gchar *as_string;
	
	g_return_if_fail (G_VALUE_HOLDS_STRING (src) &&
			  GNC_VALUE_HOLDS_NUMERIC (dest));
	
	as_string = g_value_get_string (src);
	
	numeric = g_new0 (GncNumeric, 1);
	
	string_to_gnc_numeric(const gchar* str, numeric);
	
	g_value_unset (dest);
	g_value_init (GNC_TYPE_NUMERIC);
	
	g_value_take_boxed (dest, numeric);
	
}
static void 
gnc_numeric_to_string (const GValue *src, GValue *dest) 
{
	gchar *string;
	GncNumeric *numeric;
	
	g_return_if_fail (G_VALUE_HOLDS_STRING (dest) &&
			  GNC_VALUE_HOLDS_NUMERIC (src));
			  
	numeric = gnc_value_get_numeric (src);
	
	string = gnc_numeric_to_string (numeric);
	
	g_value_set_string (dest, string);
}

GType
gnc_numeric_get_type (void)
{
	static GType type = 0;
	
	if (G_UNLIKELY (type == 0)) {
		type = g_boxed_type_register_static ("GncNumeric",
						     (GBoxedCopyFunc) gnc_numeric_copy,
						     (GBoxedFreeFunc) gnc_numeric_free);
		
		g_value_register_transform_func (G_TYPE_STRING,
						 type,
						 string_to_gnc_numeric);
		
		g_value_register_transform_func (type, 
						 G_TYPE_STRING,
						 gnc_numeric_to_string);
	}
	
	return type;
}



