/***************************************************************************
 *            gnc-split.c
 *
 *  mié ene 10 14:10:55 2007
 *  Copyright  2007  Daniel Espinosa Ortiz
 *  esodan@gmail.com
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gnc-split.h"
#include "gnc-account.h"
#include "gnc-account-private.h"
#include "Split.h"
 
static void gnc_split_class_init(GncSplitClass *klass);
static void gnc_split_init(GncSplit *sp);
static void gnc_split_finalize(GObject *object);

static void gnc_split_set_qof_book (GncObject *object, QofBook *book);
static QofBook* gnc_split_get_qof_book (GncObject *object);

struct _GncSplitPrivate {
	/* Place Private */
	Split *split;
	
	GncAccount *account;
};

typedef struct _GncSplitSignal GncSplitSignal;
typedef enum _GncSplitSignalType GncSplitSignalType;

enum _GncSplitSignalType {
	/* Signals */
	LAST_SIGNAL
};

/* properties */
enum
{
        PROP_0,
	PROP_ACCOUNT
};

struct _GncSplitSignal {
	GncSplit *object;
};

static guint gnc_split_signals[LAST_SIGNAL] = { 0 };
static GncObjectClass *parent_class = NULL;

GType
gnc_split_get_type()
{
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo our_info = {
			sizeof (GncSplitClass),
			NULL,
			NULL,
			(GClassInitFunc)gnc_split_class_init,
			NULL,
			NULL,
			sizeof (GncSplit),
			0,
			(GInstanceInitFunc)gnc_split_init,
		};

		type = g_type_register_static(G_TYPE_OBJECT, 
			"GncSplit", &our_info, 0);
	}

	return type;
}

static void
gnc_split_class_init(GncSplitClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS(klass);

	parent_class = g_type_class_peek_parent(klass);
	object_class->finalize = gnc_split_finalize;
	
	GncObjectClass *gnc_object_class = GNC_OBJECT_CLASS (klass);
	
	gnc_object_class->set_qof_book = gnc_split_set_qof_book;
	gnc_object_class->get_qof_book = gnc_split_get_qof_book;
	
	/* Install properties */
	g_object_class_install_property (object_class, PROP_ACCOUNT,
					 g_param_spec_object ("account", NULL, 
							       _("Pointer to a GncAccount object"), GNC_TYPE_ACCOUNT,
							       G_PARAM_WRITABLE));
							       
	/* Create signals here:
	   gnc_split_signals[SIGNAL_TYPE_EXAMPLE] = g_signal_new(...)
 	*/
}

static void
gnc_split_init(GncSplit *obj)
{
	obj->priv = g_new0(GncSplitPrivate, 1);
	/* Initialize private members, etc. */
	obj->priv->split = NULL;
}

static void
gnc_split_finalize(GObject *object)
{
	GncSplit *cobj;
	cobj = GNC_SPLIT(object);
	
	/* Free private members, etc. */
	
	
	g_free(cobj->priv);
	G_OBJECT_CLASS(parent_class)->finalize(object);
}

static void
gnc_split_set_property (GObject *object,
				  guint param_id,
				  const GValue *value,
				  GParamSpec *pspec)
{
	GncObject *obj;
	GncAccount *account;
	
	obj = GNC_OBJECT (object);
	account = GNC_ACCOUNT (g_value_get_object (value));
	
	switch (param_id) {
		case PROP_ACCOUNT:
			xaccSplitSetAccount (split->priv->split, 
 												gnc_account_get_qof_account (account));
 			obj->priv->account = account;
		break;
		
		default:
   			/* We don't have any other property... */
    		G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    	break;
	}
}

/**
 * gnc_split_new
 *
 * @book: A @QofBook.
 * Creates a new #GncSplit.

 * Returns: a newly created #GncSplit.
 */
GncSplit *
gnc_split_new (QofBook *book)
{
	GncSplit *obj;
	
	obj = GNC_SPLIT (g_object_new (GNC_TYPE_SPLIT, "qof_book", book, NULL));
	
	return obj;
}


/**
 * gnc_split_set_qof_book
 */
 static void
 gnc_split_set_qof_book (GncObject *object, QofBook *book)
 {
 	GncSplit *split;
 	
 	g_return_if_fail (GNC_IS_SPLIT (object));
 	g_return_if_fail (book == NULL);
	
	split = GNC_SPLIT (object);
	
	split->priv->split = xaccMallocSplit (book);
 }

/**
 * gnc_split_get_qof_book
 */
 static QofBook*
 gnc_split_get_qof_book (GncObject *object)
 {
 	GncSplit *split;
 	
 	g_return_if_fail (GNC_IS_SPLIT (object));
	
	split = GNC_SPLIT (object);
	
	return xaccSplitGetBook (split->priv->split);
 }

/**
 * gnc_split_delete
 *
 * @split: A #GncSplit
 *
 * Deletes the split.
 */
gboolean
gnc_split_delete (GncSplit *split)
{
	return	xaccSplitDestroy (split->priv->split);
}

/**
 * gnc_split_set_account
 *
 * @split: A #GncSplit
 * @account: A #GncAccount.
 *
 * Sets an #GncAccount to a #GncSplit.
 */
 void
 gnc_split_set_account (GncSplit *split, GncAccount *account)
 {
 	g_return_if_fail (GNC_IS_SPLIT (object));
 	
 	g_object_set (G_OBJECT (split), "qof_account", account);
 }
 
/**
 * gnc_split_get_account
 *
 * @split: A #GncSplit
 *
 * Returns: a the  #GncAccount.
 */
GncAccount*
gnc_split_get_account (GncSplit *split, GncAccount *account)
{
 	GncAccount *account;
 	
 	g_return_if_fail (GNC_IS_SPLIT (object));
 	
 	xaccSplitGetAccount (split->priv->split);
}
 
/**
 * gnc_split_new
 *
 * @book: A @QofBook.
 * Creates a new #GncSplit.

 * Returns: a newly created #GncSplit.
 */
GncNumeric 
gnc_split_convert_amount (const GncSplit *split)
{
	return (GncNumeric) xaccSplitConvertAmount (split->priv->split, 
																gnc_account_get_qof_account (split->priv->account));
}
